@@ -1,8 +1,10 @@
 """Coverage plugin for pytest."""
 import os
+import warnings
 
 import pytest
 import argparse
+
 from coverage.misc import CoverageException
 
 from . import embed
@@ -98,14 +100,6 @@ def pytest_load_initial_conftests(early_config, parser, args):
         early_config.pluginmanager.register(plugin, '_cov')
 
 
-def pytest_configure(config):
-    """Activate coverage plugin if appropriate."""
-    if config.getvalue('cov_source'):
-        if not config.pluginmanager.hasplugin('_cov'):
-            plugin = CovPlugin(config.option, config.pluginmanager, start=False)
-            config.pluginmanager.register(plugin, '_cov')
-
-
 class CovPlugin(object):
     """Use coverage package to produce code coverage reports.
 
@@ -184,7 +178,7 @@ class CovPlugin(object):
         """At session start determine our implementation and delegate to it."""
 
         if self.options.no_cov:
-            # Coverage can be disabled because it does not cooperate with debuggers well.py
+            # Coverage can be disabled because it does not cooperate with debuggers well.
             self._disabled = True
             return
 
@@ -238,9 +232,14 @@ class CovPlugin(object):
             try:
                 self.cov_total = self.cov_controller.summary(self.cov_report)
             except CoverageException as exc:
-                raise pytest.UsageError(
-                    'Failed to generate report: %s\n' % exc
-                )
+                message = 'Failed to generate report: %s\n' % exc
+                session.config.pluginmanager.getplugin("terminalreporter").write(
+                    'WARNING: %s\n' % message, red=True, bold=True)
+                if pytest.__version__ >= '3.8':
+                    warnings.warn(pytest.PytestWarning(message))
+                else:
+                    session.config.warn(code='COV-2', message=message)
+                self.cov_total = 0
             assert self.cov_total is not None, 'Test coverage should never be `None`'
             if self._failed_cov_total():
                 # make sure we get the EXIT_TESTSFAILED exit code
@@ -248,9 +247,12 @@ class CovPlugin(object):
 
     def pytest_terminal_summary(self, terminalreporter):
         if self._disabled:
-            msg = 'Coverage disabled via --no-cov switch!'
-            terminalreporter.write('WARNING: %s\n' % msg, red=True, bold=True)
-            terminalreporter.config.warn(code='COV-U1', message=msg)
+            message = 'Coverage disabled via --no-cov switch!'
+            terminalreporter.write('WARNING: %s\n' % message, red=True, bold=True)
+            if pytest.__version__ >= '3.8':
+                warnings.warn(pytest.PytestWarning(message))
+            else:
+                terminalreporter.config.warn(code='COV-1', message=message)
             return
         if self.cov_controller is None:
             return
@@ -264,19 +266,19 @@ class CovPlugin(object):
         if self.options.cov_fail_under is not None and self.options.cov_fail_under > 0:
             if self.cov_total < self.options.cov_fail_under:
                 markup = {'red': True, 'bold': True}
-                msg = (
+                message = (
                     'FAIL Required test coverage of %d%% not '
                     'reached. Total coverage: %.2f%%\n'
                     % (self.options.cov_fail_under, self.cov_total)
                 )
             else:
                 markup = {'green': True}
-                msg = (
+                message = (
                     'Required test coverage of %d%% '
                     'reached. Total coverage: %.2f%%\n'
                     % (self.options.cov_fail_under, self.cov_total)
                 )
-            terminalreporter.write(msg, **markup)
+            terminalreporter.write(message, **markup)
 
     def pytest_runtest_setup(self, item):
         if os.getpid() != self.pid:
@@ -289,6 +291,21 @@ class CovPlugin(object):
             embed.cleanup(self.cov)
             self.cov = None
 
+    @compat.hookwrapper
+    def pytest_runtest_call(self, item):
+        if (item.get_closest_marker('no_cover')
+                or 'no_cover' in getattr(item, 'fixturenames', ())):
+            self.cov_controller.pause()
+            yield
+            self.cov_controller.resume()
+        else:
+            yield
+
+
+@pytest.fixture
+def no_cover():
+    pass
+
 
 @pytest.fixture
 def cov(request):
@@ -300,3 +317,7 @@ def cov(request):
         if plugin.cov_controller:
             return plugin.cov_controller.cov
     return None
+
+
+def pytest_configure(config):
+    config.addinivalue_line("markers", "no_cover: disable coverage for this test.")
